###########################################################
# This is the main file driving the data preparation for the model 
# Written by  John-Mary Matovu
#             Research Fellow
#             Research for Transformation and Development
#             Kampala, Uganda
#             www.rtdug.org
###########################################################

using Ipopt, MAT, XLSX, LinearAlgebra, NamedArrays, DataStructures, UnPack
using JuMP, Complementarity
using DataFrames, Printf
using JLD2, FileIO 
#using PATHSolver
#PATHSolver.c_api_License_SetString("2830898829&Courtesy&&&USR&45321&5_1_2021&1000&PATH&GEN&31_12_2025&0_0_0&6000&0_0")


#  Provide country names for the analysis

# Call the function to balance the SAM entries at this stage
include("datastruct.jl")
include("sets.jl") 
s=sets();
include("sambalancing.jl")

# Specifiy country or region of analysis at this point
filedata=joinpath(@__DIR__, "..", "data", "data.xlsx")
dfc = DataFrame(XLSX.readtable(filedata, "Countries"))

CtryName=Dict(); [CtryName[dfc.CtryNum[i]]=dfc.CtryName[i] for i in 1:length(dfc.CtryNum)]
CtryCode=Dict(); [CtryCode[dfc.CtryNum[i]]=dfc.CtryCode[i] for i in 1:length(dfc.CtryCode)]
Africa=Dict(); [Africa[dfc.CtryNum[i]]=dfc.Africa[i] for i in 1:length(dfc.CtryCode)]
SSA=Dict(); [SSA[dfc.CtryNum[i]]=dfc.SSA[i] for i in 1:length(dfc.CtryCode)]
EastAfrica=Dict(); [EastAfrica[dfc.CtryNum[i]]=dfc.EA[i] for i in 1:length(dfc.CtryCode)]
OneCtry=Dict(); [OneCtry[dfc.CtryNum[i]]=dfc.OneCtry[i] for i in 1:length(dfc.CtryCode)]
##
# Set region of anaylysis at this point
#CtryAnal=EastAfrica;
#CtryAnal=Africa;
CtryAnal=OneCtry;

# Extract and set up Social Accounting Matrices for the Countries of Analysis
include("data.jl")

d=data(s,CtryCode,CtryAnal)

include("shares.jl")

include("varparam.jl")

include("elasticities.jl")

NCOUNTRIES=length(CtryCode);

#firstyear=2016; lastyear=2040;
res=Dict();
calib=Dict();
Countries=String[]
for i in 1:NCOUNTRIES
    if CtryAnal[i]==1    
            Ctry=CtryCode[i]
            push!(Countries,Ctry)
            println("####################")
            println(" Working on Country")
            println(Ctry)
            println("####################")
            calib[Ctry]=shares(s,d,Ctry)         
    end
end

save("calib.jld2", "data", calib)
save("countries.jld2","data",Countries)